/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_DUMPER_HH_
# define PCAP_DUMPER_HH_

extern "C"
{
# include <inttypes.h>
}
# include <string>
# include <vector>
# include <fstream>
# include <cstdlib>

# include <boost/shared_ptr.hpp>

# include <wipal/pcap/simple_types.hh>
# include <wipal/pcap/frame_descriptor.hh>
# include <wipal/tool/static_base.hh>

namespace pcapxx
{

  template <class B>
  struct descriptor;

  /**
   * Enable frame/packet dumping to PCAP files.
   *
   * A PCAP %dumper allows you to dump frames/packets into a file.
   */
  struct dumper
  {
    dumper(const std::string&	filename,
	   int32_t		type,
	   int32_t		snaplen);

    /// Dump a frame/packet given a PCAP header and some data.
    void			operator () (const pkthdr*, const void* pkt);

    /// Dump a frame/packet given a frame %descritpor.
    template <class D> void	operator () (const frame_descriptor<D>&);

    /// Retrieve a descriptor to the dumped file.
    descriptor<tool::bottom>	desc() const;

  private:
    std::vector<std::streampos>		marks_;

    std::string				file_name_;
    unsigned				pkt_count_;
    boost::shared_ptr<std::ofstream>	output_;
  };

} // End of namespace pcapxx.

# include "dumper.hxx"

#endif // ! PCAP_DESCRITPOR_HH_
