/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef AVS_HEADER_HH_
# define AVS_HEADER_HH_

# include <wipal/phy/phy.hh>

// Forward declarations.
namespace avs
{
  struct	header;
}


// Virtual types.
namespace tool
{

  template <>
  struct types<avs::header>
  {
    typedef phy::uint64_time<>	time_type;
  };

} // End of namespace tool.


// Core declarations.

/// AVS %header related compounds.
namespace avs
{

  /// AVS %header format.
  struct header: public phy::header<avs::header>
  {

    /// @name Static hierarchies related types and methods.
    //@{
    typedef avs::header				exact_type;
    typedef WP_TYPE_(time_type, exact_type)	time_type;

    size_t	len_impl(size_t, bool) const;
    time_type	time_get_impl(bool) const;
    void	time_set_impl(const time_type&, bool);

    template <class D1, class D2>
    static bool	eq_time(const pcapxx::frame_descriptor<D1>&	lhs,
			const pcapxx::frame_descriptor<D2>&	rhs,
			const unsigned				prec,
			const tool::endian::endianness		phy_end);
    //@}

    /// Get the mactime field after endianness adjustments.
    uint64_t		mactime_get() const;

    /// Get the hosttime field after endianness adjustments.
    uint64_t		hosttime_get() const;

  private:
    static uint64_t	uint64_get(const void*);

  public:

    /// Extra fields when header's version is 2.
    struct v2_extra_fields
    {

      uint32_t	sequence;		///<< Receive sequence.
      uint32_t	drops;			///<< Known dropped frames.
      uint8_t	receiver_addr[6];	///<< Receiver address.
      uint8_t	pad[2];			///<< Padding bytes.

    } __attribute__ ((__packed__));

    uint32_t		version;	///<< Header revision.
    uint32_t		length;		///<< Header length.
    uint64_t		mactime;	///<< MAC timestamp.
    uint64_t		hosttime;	///<< Host timestamp.
    uint32_t		phytype;	///<< PHY type (i.e. modulation).
    uint32_t		channel;	///<< PHY channel (frequency).
    uint32_t		datarate;	///<< PHY data rate.
    uint32_t		antenna;	///<< Antenna.
    uint32_t		priority;	///<< Priority.
    uint32_t		ssi_type;	///<< SSI type.
    uint32_t		ssi_signal;	///<< SSI signal.
    uint32_t		ssi_noise;	///<< SSI noise.
    uint32_t		preamble;	///<< Preamble type (short, long).
    uint32_t		encoding;	///<< PHY encoding type (e.g. QPSK).
    v2_extra_fields	v2_fields[];

  } __attribute__ ((__packed__));

} // End of namespace avs.

# include "avs_header.hxx"

#endif // ! AVS_HEADER_HH_
