/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_EXCEPTIONS_HH_
# define TOOL_EXCEPTIONS_HH_

# include <stdexcept>
# include <string>

namespace tool
{

  /// Exception: user manually interrupted execution.
  struct user_interruption: public std::runtime_error
  {
    user_interruption(const std::string&);
  };

  /// Exception: a file-related operation failed.
  struct file_error: public std::runtime_error
  {
    file_error(const std::string&);
  };

  /// Exception: read failed.
  struct read_error: public file_error
  {
    read_error(const std::string&);
  };

  /// Exception: write failed.
  struct write_error: public file_error
  {
    write_error(const std::string&);
  };

  /// Exception: seek failed.
  struct seek_error: public file_error
  {
    seek_error(const std::string&);
  };

  /// Exception: file contains invalid/inconsistent data.
  struct bad_file: public file_error
  {
    bad_file(const std::string&);
  };

} // End of namespace tool.

# include "exceptions.hxx"

#endif // ! TOOL_EXCEPTIONS_HH_
