/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_SSID_STATS_HXX_
# define WIFI_FRAME_STATS_SSID_STATS_HXX_

# include <boost/format.hpp>

# include <wipal/wifi/frame/stats/ssid_stats.hh>
# include <wipal/wifi/essid.hh>
# include <wipal/tool/less.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      inline
      ssid_stats::ssid_stats(const void*): total_ (0)
      {
      }

      template <class Frame>
      void
      ssid_stats::account_beacon(const Frame&		,
				 const addr* const&	,
				 const addr* const&	,
				 const optional_string&	ssid,
				 const bool&		)
      {
	if (ssid)
	  {
	    ++total_;
	    ++freqs_[*ssid];
	  }
      }

      inline
      std::ostream&
      ssid_stats::print(std::ostream& os) const
      {
	typedef std::set<freq_type::const_iterator,
			 tool::less_using_second>	sorted_ssids;

	boost::format	f ("    %40d  %f  %d\n");
	sorted_ssids	s;

	for (freq_type::const_iterator i = freqs_.begin();
	     i != freqs_.end();
	     ++i)
	  s.insert(i);

	os << "begin SSID figures\n"
	   << "  distinct SSID's: " << freqs_.size() << '\n'
	   << '\n'
	   << "  begin SSID list (ordered by frequency)\n";
	for (sorted_ssids::const_iterator i = s.begin(); i != s.end(); ++i)
	  os << (f % essid ((*i)->first)
		   % (double ((*i)->second) / double (total_))
		   % (*i)-> second);
	os << "  end SSID list (ordered by frequency)\n"
	   << '\n'
	   << "  begin SSID list (ordered by SSID)\n";
	for (freq_type::const_iterator i = freqs_.begin();
	     i != freqs_.end();
	     ++i)
	  os << (f % essid (i->first)
		   % (double (i->second) / double (total_))
		   % i-> second);
	os << "  end SSID list (ordered by SSID)\n";

	return os << "end SSID figures" << std::endl;
      }

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_STATS_SSID_STATS_HXX_
