/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TRACES_SIMILARITY_HH_
# define WIFI_TRACES_SIMILARITY_HH_

# include <set>

# include <boost/tuple/tuple.hpp>

# include <wipal/tool/options.hh>

namespace wifi
{

  /// Algorithms on IEEE 802.11 traces.
  namespace traces
  {

    enum
      {
	/// Maximum number of unique frames to process when scoring
	/// individual traces for similarity().
	similarity_score_limit = 250000,
      };

    template <class UniqueId,
	      class PhyHeader,
	      template <class, class, class> class Intersector,
	      class C>
    std::set< boost::tuple<unsigned, typename C::iterator,
				     typename C::iterator> >
    similarity(C& traces, tool::opt::list& options);

  } // End of namespace wifi::traces.

} // End of namespace wifi.

# include "similarity.hxx"

#endif // ! WIFI_TRACES_SIMILARITY_HH_
